@@ -2,14 +2,15 @@ package MCP::Constants;
 use Mojo::Base 'Exporter';
 
 use constant {
-  INVALID_PARAMS   => -32602,
-  INVALID_REQUEST  => -32600,
-  METHOD_NOT_FOUND => -32601,
-  PARSE_ERROR      => -32700,
-  PROTOCOL_VERSION => '2025-06-18'
+  INVALID_PARAMS     => -32602,
+  INVALID_REQUEST    => -32600,
+  METHOD_NOT_FOUND   => -32601,
+  PARSE_ERROR        => -32700,
+  PROTOCOL_VERSION   => $ENV{MOJO_MCP_VERSION} || '2025-11-25',
+  RESOURCE_NOT_FOUND => -32002
 };
 
-our @EXPORT_OK = qw(INVALID_PARAMS INVALID_REQUEST METHOD_NOT_FOUND PARSE_ERROR PROTOCOL_VERSION);
+our @EXPORT_OK = qw(INVALID_PARAMS INVALID_REQUEST METHOD_NOT_FOUND PARSE_ERROR PROTOCOL_VERSION RESOURCE_NOT_FOUND);
 
 1;
 
@@ -51,6 +52,10 @@ The error code for a parse error.
 
 The version of the Model Context Protocol being used.
 
+=head2 RESOURCE_NOT_FOUND
+
+The error code for a resource that was not found.
+
 =head1 SEE ALSO
 
 L<MCP>, L<https://mojolicious.org>, L<https://modelcontextprotocol.io>.
